import { Router } from 'express'
import health from '@cloudnative/health-connect'
import * as redis from '../redis.js'
import { getLatestVersion } from '../version.js'
import { once } from '../util.js'

const router = Router()
const healthCheck = new health.HealthChecker()

const redisReady = new health.ReadinessCheck('Redis ready', redis.isReady)
healthCheck.registerReadinessCheck(redisReady)

const startupCheck = new health.StartupCheck('check latest version', once(async function () {
  await getLatestVersion()
}))
healthCheck.registerStartupCheck(startupCheck)

router.use('/live', health.LivenessEndpoint(healthCheck))
router.use('/ready', health.ReadinessEndpoint(healthCheck))
router.use('/healthy', health.HealthEndpoint(healthCheck))

export default router
